import pytest
from cryptography.fernet import Fernet
from django.test import override_settings

from w.services.technical.backends.encryption.fernet_backend import FernetBackend
from w.tests.mixins.testcase_mixin import TestCaseMixin

FERNET_KEY = "ROddi992gyDMwfA3JeQko4od0iMigxPG5jpnEWyNvVw="


class TestFernetBackend(TestCaseMixin):
    @classmethod
    def setup_class(cls):
        super().setup_class()
        cls.fernet = Fernet(FERNET_KEY)
        cls.to_encrypt = "ma chaine à crypter"
        cls.backend = FernetBackend

    def setup_method(self):
        FernetBackend.reset()

    """
    encrypt
    """

    def test_encrypt_with_missing_settings_raises_runtime_error(self):
        match = "ENCRYPTION_FERNET_KEY is missing in your settings"
        with pytest.raises(RuntimeError, match=match):
            self.backend.encrypt(self.to_encrypt)

    @override_settings(ENCRYPTION_FERNET_KEY=None)
    def test_encrypt_with_empty_settings_raises_runtime_error(self):
        match = "ENCRYPTION_FERNET_KEY is empty, you must set a Fernet key"
        with pytest.raises(RuntimeError, match=match):
            self.backend.encrypt(self.to_encrypt)

    @override_settings(ENCRYPTION_FERNET_KEY=FERNET_KEY)
    def test_encrypt_with_success_return_str(self):
        actual = self.backend.encrypt(self.to_encrypt)
        assert self.to_encrypt == self.fernet.decrypt(actual.encode()).decode()

    """
    decrypt
    """

    def test_decrypt_with_missing_settings_raises_runtime_error(self):
        match = "ENCRYPTION_FERNET_KEY is missing in your settings"
        with pytest.raises(RuntimeError, match=match):
            self.backend.decrypt("fakecryptedmessage")

    @override_settings(ENCRYPTION_FERNET_KEY=FERNET_KEY)
    def test_decrypt_with_success_return_str(self):
        crypted = self.fernet.encrypt(self.to_encrypt.encode()).decode()
        actual = self.backend.decrypt(crypted)
        assert actual == self.to_encrypt
