import pytest
from django.test import override_settings

from w.services.technical.backends.hasher.blake2_backend import Blake2Backend
from w.tests.mixins.testcase_mixin import TestCaseMixin


class TestBlake2Backend(TestCaseMixin):
    @classmethod
    def setup_class(cls):
        super().setup_class()
        cls.message = "message-to-hash"
        cls.backend = Blake2Backend

    def setup_method(self):
        self.backend.reset()

    """
    hash
    """

    def test_encrypt_with_missing_settings_raises_runtime_error(self):
        match = "HASH_SECRET_KEY is missing in your settings"
        with pytest.raises(RuntimeError, match=match):
            self.backend.hash(self.message)

    @override_settings(HASH_SECRET_KEY=None)
    def test_encrypt_with_empty_secret_key_raises_runtime_error(self):
        match = "HASH_SECRET_KEY is empty, you must set a hash secret key"
        with pytest.raises(RuntimeError, match=match):
            self.backend.hash(self.message)

    @override_settings(HASH_SECRET_KEY="secret-key-should-be-32bits")
    def test_hash_succeed(self):
        actual = self.backend.hash(self.message)
        expected = (
            "b9e6d26de56a25f13a5fbe87be630f4e5a06797ee4431156ddc3837a9a75fdcacbd28a"
            "6fd255547e5095f2d5850020faf8b36b2ec1b8f729868c206fcceee9f5"
        )
        assert actual == expected
